@@ -1,8 +1,14 @@
-use std::{env, ffi::OsString, fmt, iter::Peekable, mem};
+use std::{
+    env,
+    ffi::{OsStr, OsString},
+    fmt,
+    iter::Peekable,
+    mem,
+};
 
 use anyhow::{bail, format_err, Error, Result};
 
-use crate::{rustup, term, Cargo, Feature, Rustup};
+use crate::{term, Feature, Rustup};
 
 pub(crate) struct Args<'a> {
     pub(crate) leading_args: Vec<&'a str>,
@@ -34,8 +40,12 @@ pub(crate) struct Args<'a> {
     pub(crate) clean_per_run: bool,
     /// --clean-per-version
     pub(crate) clean_per_version: bool,
-    /// --version-range and --version-step
-    pub(crate) version_range: Option<Vec<String>>,
+    /// --keep-going
+    pub(crate) keep_going: bool,
+    /// --version-range
+    pub(crate) version_range: Option<&'a str>,
+    /// --version-step
+    pub(crate) version_step: Option<&'a str>,
 
     // options for --each-feature and --feature-powerset
     /// --optional-deps [DEPS]...
@@ -64,11 +74,9 @@ pub(crate) struct Args<'a> {
     /// --features <FEATURES>...
     pub(crate) features: Vec<&'a str>,
 
-    // propagated (as a part of leading_args) to cargo
+    // propagated to cargo (as a part of leading_args)
     /// --no-default-features
     pub(crate) no_default_features: bool,
-    /// -v, --verbose, -vv
-    pub(crate) verbose: bool,
     // Note: specifying multiple `--target` flags requires unstable `-Z multitarget`,
     // so cargo-hack currently only supports a single `--target`.
     /// --target <TRIPLE>...
@@ -93,7 +101,7 @@ fn handle_args(args: impl IntoIterator<Item = impl Into<OsString>>) -> Result<Ve
         .collect()
 }
 
-pub(crate) fn parse_args<'a>(raw: &'a [String]) -> Result<(Args<'a>, Cargo)> {
+pub(crate) fn parse_args<'a>(raw: &'a [String], cargo: &OsStr) -> Result<Args<'a>> {
     let mut iter = raw.iter();
     let args = &mut iter.by_ref().map(String::as_str).peekable();
     match args.next() {
@@ -124,6 +132,7 @@ pub(crate) fn parse_args<'a>(raw: &'a [String]) -> Result<(Args<'a>, Cargo)> {
     let mut ignore_unknown_features = false;
     let mut clean_per_run = false;
     let mut clean_per_version = false;
+    let mut keep_going = false;
     let mut version_range = None;
     let mut version_step = None;
 
@@ -255,6 +264,7 @@ pub(crate) fn parse_args<'a>(raw: &'a [String]) -> Result<(Args<'a>, Cargo)> {
                 "--include-deps-features" => parse_flag!(include_deps_features),
                 "--clean-per-run" => parse_flag!(clean_per_run),
                 "--clean-per-version" => parse_flag!(clean_per_version),
+                "--keep-going" => parse_flag!(keep_going),
                 "--ignore-unknown-features" => parse_flag!(ignore_unknown_features),
                 // allow multiple uses
                 "--verbose" | "-v" | "-vv" => {
@@ -357,8 +367,6 @@ pub(crate) fn parse_args<'a>(raw: &'a [String]) -> Result<(Args<'a>, Cargo)> {
             v.push(Feature::group(g));
             Ok(v)
         })?;
-    let version_range =
-        version_range.map(|range| rustup::version_range(range, version_step)).transpose()?;
 
     if let Some(subcommand) = subcommand {
         match subcommand {
@@ -448,9 +456,7 @@ pub(crate) fn parse_args<'a>(raw: &'a [String]) -> Result<(Args<'a>, Cargo)> {
             print_version();
             std::process::exit(0);
         } else if leading.contains(&"--list") {
-            let mut line = Cargo::new().process();
-            line.arg("--list");
-            line.run()?;
+            cmd!(cargo, "--list").run()?;
             std::process::exit(0);
         } else if !remove_dev_deps {
             // TODO: improve this
@@ -458,12 +464,8 @@ pub(crate) fn parse_args<'a>(raw: &'a [String]) -> Result<(Args<'a>, Cargo)> {
         }
     }
 
-    let cargo = Cargo::new();
     let rustup = Rustup::new();
 
-    if cargo.version < 41 && include_deps_features {
-        bail!("--include-deps-features requires Cargo 1.41 or later");
-    }
     if rustup.version < 23 && version_range.is_some() {
         bail!("--version-range requires rustup 1.23 or later");
     }
@@ -474,49 +476,72 @@ pub(crate) fn parse_args<'a>(raw: &'a [String]) -> Result<(Args<'a>, Cargo)> {
         );
     }
 
+    // https://github.com/taiki-e/cargo-hack/issues/42
+    // https://github.com/rust-lang/cargo/pull/8799
+    let namespaced_features = has_z_flag(&leading, "namespaced-features");
     exclude_no_default_features |= !include_features.is_empty();
-    exclude_all_features |= !include_features.is_empty() || !exclude_features.is_empty();
+    exclude_all_features |= !include_features.is_empty()
+        || !exclude_features.is_empty()
+        || (feature_powerset && !namespaced_features && depth.is_none());
     exclude_features.extend_from_slice(&features);
 
-    Ok((
-        Args {
-            leading_args: leading,
-            trailing_args: iter.as_slice(),
+    term::set_verbose(verbose);
+    Ok(Args {
+        leading_args: leading,
+        trailing_args: iter.as_slice(),
 
-            subcommand,
+        subcommand,
 
-            manifest_path,
-            package,
-            exclude,
-            workspace: workspace.is_some(),
-            each_feature,
-            feature_powerset,
-            no_dev_deps,
-            remove_dev_deps,
-            ignore_private,
-            ignore_unknown_features,
-            optional_deps,
-            clean_per_run,
-            clean_per_version,
-            include_features: include_features.into_iter().map(Into::into).collect(),
-            include_deps_features,
-            version_range,
+        manifest_path,
+        package,
+        exclude,
+        workspace: workspace.is_some(),
+        each_feature,
+        feature_powerset,
+        no_dev_deps,
+        remove_dev_deps,
+        ignore_private,
+        ignore_unknown_features,
+        optional_deps,
+        clean_per_run,
+        clean_per_version,
+        keep_going,
+        include_features: include_features.into_iter().map(Into::into).collect(),
+        include_deps_features,
+        version_range,
+        version_step,
 
-            depth,
-            group_features,
+        depth,
+        group_features,
 
-            exclude_features,
-            exclude_no_default_features,
-            exclude_all_features,
+        exclude_features,
+        exclude_no_default_features,
+        exclude_all_features,
 
-            features,
+        features,
 
-            no_default_features,
-            verbose,
-            target,
-        },
-        cargo,
-    ))
+        no_default_features,
+        target,
+    })
+}
+
+fn has_z_flag(args: &[&str], name: &str) -> bool {
+    let mut iter = args.iter().copied();
+    while let Some(mut arg) = iter.next() {
+        if arg == "-Z" {
+            arg = iter.next().unwrap();
+        } else if let Some(a) = arg.strip_prefix("-Z") {
+            arg = a
+        } else {
+            continue;
+        }
+        if let Some(rest) = arg.strip_prefix(name) {
+            if rest.is_empty() || rest.starts_with('=') {
+                return true;
+            }
+        }
+    }
+    false
 }
 
 fn parse_opt<'a>(
@@ -564,27 +589,40 @@ const HELP: &[HelpText<'_>] = &[
     ("", "--manifest-path", "<PATH>", "Path to Cargo.toml", &[]),
     ("", "--features", "<FEATURES>...", "Space-separated list of features to activate", &[]),
     ("", "--each-feature", "", "Perform for each feature of the package", &[
-        "This also includes runs with just --no-default-features flag, --all-features flag, and default features.",
+        "This also includes runs with just --no-default-features flag, and default features.",
+        "When this flag is not used together with --exclude-features (--skip) and \
+         --include-features and there are multiple features, this also includes runs with \
+         just --all-features flag."
     ]),
     ("", "--feature-powerset", "", "Perform for the feature powerset of the package", &[
-        "This also includes runs with just --no-default-features flag, --all-features flag, and default features.",
+        "This also includes runs with just --no-default-features flag, and default features.",
+        // https://github.com/rust-lang/cargo/pull/8799
+        "When this flag is used together with --depth or namespaced features \
+         (-Z namespaced-features) and not used together with --exclude-features (--skip) and \
+         --include-features and there are multiple features, this also includes runs with just \
+         --all-features flag."
     ]),
     ("", "--optional-deps", "[DEPS]...", "Use optional dependencies as features", &[
         "If DEPS are not specified, all optional dependencies are considered as features.",
-        "This flag can only be used together with either --each-feature flag or --feature-powerset flag.",
+        "This flag can only be used together with either --each-feature flag or --feature-powerset \
+         flag.",
     ]),
     ("", "--skip", "<FEATURES>...", "Alias for --exclude-features", &[]),
     ("", "--exclude-features", "<FEATURES>...", "Space-separated list of features to exclude", &[
         "To exclude run of default feature, using value `--exclude-features default`.",
-        "To exclude run of just --no-default-features flag, using --exclude-no-default-features flag.",
+        "To exclude run of just --no-default-features flag, using --exclude-no-default-features \
+         flag.",
         "To exclude run of just --all-features flag, using --exclude-all-features flag.",
-        "This flag can only be used together with either --each-feature flag or --feature-powerset flag.",
+        "This flag can only be used together with either --each-feature flag or --feature-powerset \
+         flag.",
     ]),
     ("", "--exclude-no-default-features", "", "Exclude run of just --no-default-features flag", &[
-        "This flag can only be used together with either --each-feature flag or --feature-powerset flag.",
+        "This flag can only be used together with either --each-feature flag or --feature-powerset \
+         flag.",
     ]),
     ("", "--exclude-all-features", "", "Exclude run of just --all-features flag", &[
-        "This flag can only be used together with either --each-feature flag or --feature-powerset flag.",
+        "This flag can only be used together with either --each-feature flag or --feature-powerset \
+         flag.",
     ]),
     (
         "",
@@ -597,26 +635,31 @@ const HELP: &[HelpText<'_>] = &[
         ],
     ),
     ("", "--group-features", "<FEATURES>...", "Space-separated list of features to group", &[
-        "To specify multiple groups, use this option multiple times: `--group-features a,b --group-features c,d`",
+        "To specify multiple groups, use this option multiple times: `--group-features a,b \
+         --group-features c,d`",
         "This flag can only be used together with --feature-powerset flag.",
     ]),
     (
         "",
         "--include-features",
         "<FEATURES>...",
-        "Include only the specified features in the feature combinations instead of package features",
+        "Include only the specified features in the feature combinations instead of package \
+         features",
         &[
-            "This flag can only be used together with either --each-feature flag or --feature-powerset flag.",
+            "This flag can only be used together with either --each-feature flag or \
+             --feature-powerset flag.",
         ],
     ),
     ("", "--no-dev-deps", "", "Perform without dev-dependencies", &[
-        "Note that this flag removes dev-dependencies from real `Cargo.toml` while cargo-hack is running and restores it when finished.",
+        "Note that this flag removes dev-dependencies from real `Cargo.toml` while cargo-hack is \
+         running and restores it when finished.",
     ]),
     (
         "",
         "--remove-dev-deps",
         "",
-        "Equivalent to --no-dev-deps flag except for does not restore the original `Cargo.toml` after performed",
+        "Equivalent to --no-dev-deps flag except for does not restore the original `Cargo.toml` \
+         after performed",
         &[],
     ),
     ("", "--ignore-private", "", "Skip to perform on `publish = false` packages", &[]),
@@ -633,7 +676,8 @@ const HELP: &[HelpText<'_>] = &[
         "<START>..[END]",
         "Perform commands on a specified (inclusive) range of Rust versions",
         &[
-            "If the given range is unclosed, the latest stable compiler is treated as the upper bound.",
+            "If the given range is unclosed, the latest stable compiler is treated as the upper \
+             bound.",
             "Note that ranges are always inclusive ranges.",
         ],
     ),
@@ -645,13 +689,15 @@ const HELP: &[HelpText<'_>] = &[
         &["This flag can only be used together with --version-range flag."],
     ),
     ("", "--clean-per-run", "", "Remove artifacts for that package before running the command", &[
-        "If used this flag with --workspace, --each-feature, or --feature-powerset, artifacts will be removed before each run.",
+        "If used this flag with --workspace, --each-feature, or --feature-powerset, artifacts will \
+         be removed before each run.",
         "Note that dependencies artifacts will be preserved.",
     ]),
     ("", "--clean-per-version", "", "Remove artifacts per Rust version", &[
         "Note that dependencies artifacts will also be removed.",
         "This flag can only be used together with --version-range flag.",
     ]),
+    ("", "--keep-going", "", "Keep going on failure", &[]),
     ("-v", "--verbose", "", "Use verbose output", &[]),
     ("", "--color", "<WHEN>", "Coloring: auto, always, never", &[
         "This flag will be propagated to cargo.",
@@ -881,7 +927,6 @@ mod tests {
     use std::{env, panic, path::Path, process::Command};
 
     use anyhow::Result;
-    use tempfile::Builder;
 
     use super::Help;
     use crate::fs;
@@ -911,16 +956,18 @@ mod tests {
         let expected = fs::read_to_string(expected_path).unwrap();
         if expected != actual {
             if env::var_os("CI").is_some() {
-                let outdir = Builder::new().prefix("assert_diff").tempdir().unwrap();
+                let outdir = tempfile::tempdir().unwrap();
                 let actual_path = &outdir.path().join(expected_path.file_name().unwrap());
                 fs::write(actual_path, actual).unwrap();
                 let status = Command::new("git")
                     .args(&["--no-pager", "diff", "--no-index", "--"])
-                    .args(&[expected_path, actual_path])
+                    .args(&[expected_path.strip_prefix(manifest_dir).unwrap(), actual_path])
+                    .current_dir(manifest_dir)
                     .status()
                     .unwrap();
                 assert!(!status.success());
-                panic!("assertion failed");
+                // patch -p1 <<'EOF' ... EOF
+                panic!("assertion failed; please run test locally and commit resulting changes, or apply above diff as patch");
             } else {
                 fs::write(expected_path, actual).unwrap();
             }
