@@ -26,7 +26,7 @@ pub fn cargo_bin_exe() -> Command {
 
 fn test_toolchain() -> String {
     if let Some(toolchain) = test_version() {
-        format!(" +1.{}", toolchain)
+        format!("+1.{} ", toolchain)
     } else {
         String::new()
     }
@@ -45,6 +45,14 @@ fn test_version() -> Option<u32> {
     *TEST_VERSION
 }
 
+pub fn has_stable_toolchain() -> bool {
+    static HAS_STABLE_TOOLCHAIN: Lazy<Option<bool>> = Lazy::new(|| {
+        let output = Command::new("rustup").args(&["toolchain", "list"]).output().ok()?;
+        Some(String::from_utf8(output.stdout).ok()?.contains("stable"))
+    });
+    HAS_STABLE_TOOLCHAIN.unwrap_or_default()
+}
+
 pub fn cargo_hack<O: AsRef<OsStr>>(args: impl AsRef<[O]>) -> Command {
     let args = args.as_ref();
     let mut cmd = cargo_bin_exe();
@@ -133,10 +141,12 @@ struct AssertOutputInner {
 
 #[track_caller]
 fn line_separated(lines: &str, f: impl FnMut(&str)) {
-    let lines = if lines.contains("`cargo +") {
+    let lines = if lines.contains("cargo +")
+        || lines.contains(&format!("cargo{} +", env::consts::EXE_SUFFIX))
+    {
         lines.to_string()
     } else {
-        lines.replace("`cargo", &format!("`cargo{}", test_toolchain()))
+        lines.replace("cargo ", &format!("cargo {}", test_toolchain()))
     };
     lines.split('\n').map(str::trim).filter(|line| !line.is_empty()).for_each(f);
 }
