@@ -1,11 +1,12 @@
 #![warn(rust_2018_idioms, single_use_lifetimes)]
-#![allow(clippy::if_then_panic)]
 
 mod auxiliary;
 
 use std::env;
 
-use auxiliary::{cargo_bin_exe, cargo_hack, target_triple, CommandExt, SEPARATOR};
+use auxiliary::{
+    cargo_bin_exe, cargo_hack, has_stable_toolchain, target_triple, CommandExt, SEPARATOR,
+};
 
 #[test]
 fn failures() {
@@ -415,23 +416,22 @@ fn each_feature_failure() {
 fn feature_powerset() {
     cargo_hack(["check", "--feature-powerset"]).assert_success("real").stderr_contains(
         "
-        running `cargo check --no-default-features` on real (1/17)
-        running `cargo check --no-default-features --features a` on real (2/17)
-        running `cargo check --no-default-features --features b` on real (3/17)
-        running `cargo check --no-default-features --features a,b` on real (4/17)
-        running `cargo check --no-default-features --features c` on real (5/17)
-        running `cargo check --no-default-features --features a,c` on real (6/17)
-        running `cargo check --no-default-features --features b,c` on real (7/17)
-        running `cargo check --no-default-features --features a,b,c` on real (8/17)
-        running `cargo check --no-default-features --features default` on real (9/17)
-        running `cargo check --no-default-features --features a,default` on real (10/17)
-        running `cargo check --no-default-features --features b,default` on real (11/17)
-        running `cargo check --no-default-features --features a,b,default` on real (12/17)
-        running `cargo check --no-default-features --features c,default` on real (13/17)
-        running `cargo check --no-default-features --features a,c,default` on real (14/17)
-        running `cargo check --no-default-features --features b,c,default` on real (15/17)
-        running `cargo check --no-default-features --features a,b,c,default` on real (16/17)
-        running `cargo check --no-default-features --all-features` on real (17/17)
+        running `cargo check --no-default-features` on real (1/16)
+        running `cargo check --no-default-features --features a` on real (2/16)
+        running `cargo check --no-default-features --features b` on real (3/16)
+        running `cargo check --no-default-features --features a,b` on real (4/16)
+        running `cargo check --no-default-features --features c` on real (5/16)
+        running `cargo check --no-default-features --features a,c` on real (6/16)
+        running `cargo check --no-default-features --features b,c` on real (7/16)
+        running `cargo check --no-default-features --features a,b,c` on real (8/16)
+        running `cargo check --no-default-features --features default` on real (9/16)
+        running `cargo check --no-default-features --features a,default` on real (10/16)
+        running `cargo check --no-default-features --features b,default` on real (11/16)
+        running `cargo check --no-default-features --features a,b,default` on real (12/16)
+        running `cargo check --no-default-features --features c,default` on real (13/16)
+        running `cargo check --no-default-features --features a,c,default` on real (14/16)
+        running `cargo check --no-default-features --features b,c,default` on real (15/16)
+        running `cargo check --no-default-features --features a,b,c,default` on real (16/16)
         ",
     );
 
@@ -440,15 +440,14 @@ fn feature_powerset() {
         .assert_success("real")
         .stderr_contains(
             "
-            running `cargo check --no-default-features --features a` on real (1/9)
-            running `cargo check --no-default-features --features a,b` on real (2/9)
-            running `cargo check --no-default-features --features a,c` on real (3/9)
-            running `cargo check --no-default-features --features a,b,c` on real (4/9)
-            running `cargo check --no-default-features --features a,default` on real (5/9)
-            running `cargo check --no-default-features --features a,b,default` on real (6/9)
-            running `cargo check --no-default-features --features a,c,default` on real (7/9)
-            running `cargo check --no-default-features --features a,b,c,default` on real (8/9)
-            running `cargo check --no-default-features --all-features --features a` on real (9/9)
+            running `cargo check --no-default-features --features a` on real (1/8)
+            running `cargo check --no-default-features --features a,b` on real (2/8)
+            running `cargo check --no-default-features --features a,c` on real (3/8)
+            running `cargo check --no-default-features --features a,b,c` on real (4/8)
+            running `cargo check --no-default-features --features a,default` on real (5/8)
+            running `cargo check --no-default-features --features a,b,default` on real (6/8)
+            running `cargo check --no-default-features --features a,c,default` on real (7/8)
+            running `cargo check --no-default-features --features a,b,c,default` on real (8/8)
             ",
         )
         .stderr_not_contains("--features a,a");
@@ -477,17 +476,16 @@ fn powerset_deduplication() {
         .assert_success2("powerset_deduplication", Some(34))
         .stderr_contains(
             "
-            running `cargo check --no-default-features` on deduplication (1/11)
-            running `cargo check --no-default-features --features a` on deduplication (2/11)
-            running `cargo check --no-default-features --features b` on deduplication (3/11)
-            running `cargo check --no-default-features --features c` on deduplication (4/11)
-            running `cargo check --no-default-features --features d` on deduplication (5/11)
-            running `cargo check --no-default-features --features a,d` on deduplication (6/11)
-            running `cargo check --no-default-features --features b,d` on deduplication (7/11)
-            running `cargo check --no-default-features --features c,d` on deduplication (8/11)
-            running `cargo check --no-default-features --features e` on deduplication (9/11)
-            running `cargo check --no-default-features --features c,e` on deduplication (10/11)
-            running `cargo check --no-default-features --all-features` on deduplication (11/11)
+            running `cargo check --no-default-features` on deduplication (1/10)
+            running `cargo check --no-default-features --features a` on deduplication (2/10)
+            running `cargo check --no-default-features --features b` on deduplication (3/10)
+            running `cargo check --no-default-features --features c` on deduplication (4/10)
+            running `cargo check --no-default-features --features d` on deduplication (5/10)
+            running `cargo check --no-default-features --features a,d` on deduplication (6/10)
+            running `cargo check --no-default-features --features b,d` on deduplication (7/10)
+            running `cargo check --no-default-features --features c,d` on deduplication (8/10)
+            running `cargo check --no-default-features --features e` on deduplication (9/10)
+            running `cargo check --no-default-features --features c,e` on deduplication (10/10)
             ",
         )
         .stderr_not_contains(
@@ -506,21 +504,20 @@ fn powerset_deduplication() {
         .assert_success2("powerset_deduplication", Some(34))
         .stderr_contains(
             "
-            running `cargo check --no-default-features` on deduplication (1/15)
-            running `cargo check --no-default-features --features a` on deduplication (2/15)
-            running `cargo check --no-default-features --features b` on deduplication (3/15)
-            running `cargo check --no-default-features --features c` on deduplication (4/15)
-            running `cargo check --no-default-features --features d` on deduplication (5/15)
-            running `cargo check --no-default-features --features a,d` on deduplication (6/15)
-            running `cargo check --no-default-features --features b,d` on deduplication (7/15)
-            running `cargo check --no-default-features --features c,d` on deduplication (8/15)
-            running `cargo check --no-default-features --features e` on deduplication (9/15)
-            running `cargo check --no-default-features --features c,e` on deduplication (10/15)
-            running `cargo check --no-default-features --features member1` on deduplication (11/15)
-            running `cargo check --no-default-features --features a,member1` on deduplication (12/15)
-            running `cargo check --no-default-features --features b,member1` on deduplication (13/15)
-            running `cargo check --no-default-features --features c,member1` on deduplication (14/15)
-            running `cargo check --no-default-features --all-features` on deduplication (15/15)
+            running `cargo check --no-default-features` on deduplication (1/14)
+            running `cargo check --no-default-features --features a` on deduplication (2/14)
+            running `cargo check --no-default-features --features b` on deduplication (3/14)
+            running `cargo check --no-default-features --features c` on deduplication (4/14)
+            running `cargo check --no-default-features --features d` on deduplication (5/14)
+            running `cargo check --no-default-features --features a,d` on deduplication (6/14)
+            running `cargo check --no-default-features --features b,d` on deduplication (7/14)
+            running `cargo check --no-default-features --features c,d` on deduplication (8/14)
+            running `cargo check --no-default-features --features e` on deduplication (9/14)
+            running `cargo check --no-default-features --features c,e` on deduplication (10/14)
+            running `cargo check --no-default-features --features member1` on deduplication (11/14)
+            running `cargo check --no-default-features --features a,member1` on deduplication (12/14)
+            running `cargo check --no-default-features --features b,member1` on deduplication (13/14)
+            running `cargo check --no-default-features --features c,member1` on deduplication (14/14)
             ",
         )
         .stderr_not_contains(
@@ -539,14 +536,13 @@ fn powerset_deduplication() {
         .assert_success2("powerset_deduplication", Some(34))
         .stderr_contains(
             "
-            running `cargo check --no-default-features` on deduplication (1/8)
-            running `cargo check --no-default-features --features a` on deduplication (2/8)
-            running `cargo check --no-default-features --features c` on deduplication (3/8)
-            running `cargo check --no-default-features --features e` on deduplication (4/8)
-            running `cargo check --no-default-features --features c,e` on deduplication (5/8)
-            running `cargo check --no-default-features --features b,d` on deduplication (6/8)
-            running `cargo check --no-default-features --features c,b,d` on deduplication (7/8)
-            running `cargo check --no-default-features --all-features` on deduplication (8/8)
+            running `cargo check --no-default-features` on deduplication (1/7)
+            running `cargo check --no-default-features --features a` on deduplication (2/7)
+            running `cargo check --no-default-features --features c` on deduplication (3/7)
+            running `cargo check --no-default-features --features e` on deduplication (4/7)
+            running `cargo check --no-default-features --features c,e` on deduplication (5/7)
+            running `cargo check --no-default-features --features b,d` on deduplication (6/7)
+            running `cargo check --no-default-features --features c,b,d` on deduplication (7/7)
             ",
         )
         .stderr_not_contains(
@@ -561,17 +557,16 @@ fn powerset_deduplication() {
         .assert_success2("powerset_deduplication", Some(34))
         .stderr_contains(
             "
-            running `cargo check --no-default-features` on deduplication (1/11)
-            running `cargo check --no-default-features --features a` on deduplication (2/11)
-            running `cargo check --no-default-features --features c` on deduplication (3/11)
-            running `cargo check --no-default-features --features e` on deduplication (4/11)
-            running `cargo check --no-default-features --features c,e` on deduplication (5/11)
-            running `cargo check --no-default-features --features member1` on deduplication (6/11)
-            running `cargo check --no-default-features --features a,member1` on deduplication (7/11)
-            running `cargo check --no-default-features --features c,member1` on deduplication (8/11)
-            running `cargo check --no-default-features --features b,d` on deduplication (9/11)
-            running `cargo check --no-default-features --features c,b,d` on deduplication (10/11)
-            running `cargo check --no-default-features --all-features` on deduplication (11/11)
+            running `cargo check --no-default-features` on deduplication (1/10)
+            running `cargo check --no-default-features --features a` on deduplication (2/10)
+            running `cargo check --no-default-features --features c` on deduplication (3/10)
+            running `cargo check --no-default-features --features e` on deduplication (4/10)
+            running `cargo check --no-default-features --features c,e` on deduplication (5/10)
+            running `cargo check --no-default-features --features member1` on deduplication (6/10)
+            running `cargo check --no-default-features --features a,member1` on deduplication (7/10)
+            running `cargo check --no-default-features --features c,member1` on deduplication (8/10)
+            running `cargo check --no-default-features --features b,d` on deduplication (9/10)
+            running `cargo check --no-default-features --features c,b,d` on deduplication (10/10)
             ",
         )
         .stderr_not_contains(
@@ -591,50 +586,49 @@ fn powerset_deduplication_include_deps_features() {
     // TODO: Since easytime/default depends on easytime/std, their combination should be excluded,
     // but it's not working yet because include-deps-features itself isn't fully implemented.
     cargo_hack(["check", "--feature-powerset", "--include-deps-features"])
-        .assert_success2("powerset_deduplication", Some(41))
+        .assert_success2("powerset_deduplication",  Some(if has_stable_toolchain() { 34 } else { 41 }))
         .stderr_contains(
             "
-            running `cargo check --no-default-features` on deduplication (1/41)
-            running `cargo check --no-default-features --features a` on deduplication (2/41)
-            running `cargo check --no-default-features --features b` on deduplication (3/41)
-            running `cargo check --no-default-features --features c` on deduplication (4/41)
-            running `cargo check --no-default-features --features d` on deduplication (5/41)
-            running `cargo check --no-default-features --features a,d` on deduplication (6/41)
-            running `cargo check --no-default-features --features b,d` on deduplication (7/41)
-            running `cargo check --no-default-features --features c,d` on deduplication (8/41)
-            running `cargo check --no-default-features --features e` on deduplication (9/41)
-            running `cargo check --no-default-features --features c,e` on deduplication (10/41)
-            running `cargo check --no-default-features --features easytime/default` on deduplication (11/41)
-            running `cargo check --no-default-features --features a,easytime/default` on deduplication (12/41)
-            running `cargo check --no-default-features --features b,easytime/default` on deduplication (13/41)
-            running `cargo check --no-default-features --features c,easytime/default` on deduplication (14/41)
-            running `cargo check --no-default-features --features d,easytime/default` on deduplication (15/41)
-            running `cargo check --no-default-features --features a,d,easytime/default` on deduplication (16/41)
-            running `cargo check --no-default-features --features b,d,easytime/default` on deduplication (17/41)
-            running `cargo check --no-default-features --features c,d,easytime/default` on deduplication (18/41)
-            running `cargo check --no-default-features --features e,easytime/default` on deduplication (19/41)
-            running `cargo check --no-default-features --features c,e,easytime/default` on deduplication (20/41)
-            running `cargo check --no-default-features --features easytime/std` on deduplication (21/41)
-            running `cargo check --no-default-features --features a,easytime/std` on deduplication (22/41)
-            running `cargo check --no-default-features --features b,easytime/std` on deduplication (23/41)
-            running `cargo check --no-default-features --features c,easytime/std` on deduplication (24/41)
-            running `cargo check --no-default-features --features d,easytime/std` on deduplication (25/41)
-            running `cargo check --no-default-features --features a,d,easytime/std` on deduplication (26/41)
-            running `cargo check --no-default-features --features b,d,easytime/std` on deduplication (27/41)
-            running `cargo check --no-default-features --features c,d,easytime/std` on deduplication (28/41)
-            running `cargo check --no-default-features --features e,easytime/std` on deduplication (29/41)
-            running `cargo check --no-default-features --features c,e,easytime/std` on deduplication (30/41)
-            running `cargo check --no-default-features --features easytime/default,easytime/std` on deduplication (31/41)
-            running `cargo check --no-default-features --features a,easytime/default,easytime/std` on deduplication (32/41)
-            running `cargo check --no-default-features --features b,easytime/default,easytime/std` on deduplication (33/41)
-            running `cargo check --no-default-features --features c,easytime/default,easytime/std` on deduplication (34/41)
-            running `cargo check --no-default-features --features d,easytime/default,easytime/std` on deduplication (35/41)
-            running `cargo check --no-default-features --features a,d,easytime/default,easytime/std` on deduplication (36/41)
-            running `cargo check --no-default-features --features b,d,easytime/default,easytime/std` on deduplication (37/41)
-            running `cargo check --no-default-features --features c,d,easytime/default,easytime/std` on deduplication (38/41)
-            running `cargo check --no-default-features --features e,easytime/default,easytime/std` on deduplication (39/41)
-            running `cargo check --no-default-features --features c,e,easytime/default,easytime/std` on deduplication (40/41)
-            running `cargo check --no-default-features --all-features` on deduplication (41/41)
+            running `cargo check --no-default-features` on deduplication (1/40)
+            running `cargo check --no-default-features --features a` on deduplication (2/40)
+            running `cargo check --no-default-features --features b` on deduplication (3/40)
+            running `cargo check --no-default-features --features c` on deduplication (4/40)
+            running `cargo check --no-default-features --features d` on deduplication (5/40)
+            running `cargo check --no-default-features --features a,d` on deduplication (6/40)
+            running `cargo check --no-default-features --features b,d` on deduplication (7/40)
+            running `cargo check --no-default-features --features c,d` on deduplication (8/40)
+            running `cargo check --no-default-features --features e` on deduplication (9/40)
+            running `cargo check --no-default-features --features c,e` on deduplication (10/40)
+            running `cargo check --no-default-features --features easytime/default` on deduplication (11/40)
+            running `cargo check --no-default-features --features a,easytime/default` on deduplication (12/40)
+            running `cargo check --no-default-features --features b,easytime/default` on deduplication (13/40)
+            running `cargo check --no-default-features --features c,easytime/default` on deduplication (14/40)
+            running `cargo check --no-default-features --features d,easytime/default` on deduplication (15/40)
+            running `cargo check --no-default-features --features a,d,easytime/default` on deduplication (16/40)
+            running `cargo check --no-default-features --features b,d,easytime/default` on deduplication (17/40)
+            running `cargo check --no-default-features --features c,d,easytime/default` on deduplication (18/40)
+            running `cargo check --no-default-features --features e,easytime/default` on deduplication (19/40)
+            running `cargo check --no-default-features --features c,e,easytime/default` on deduplication (20/40)
+            running `cargo check --no-default-features --features easytime/std` on deduplication (21/40)
+            running `cargo check --no-default-features --features a,easytime/std` on deduplication (22/40)
+            running `cargo check --no-default-features --features b,easytime/std` on deduplication (23/40)
+            running `cargo check --no-default-features --features c,easytime/std` on deduplication (24/40)
+            running `cargo check --no-default-features --features d,easytime/std` on deduplication (25/40)
+            running `cargo check --no-default-features --features a,d,easytime/std` on deduplication (26/40)
+            running `cargo check --no-default-features --features b,d,easytime/std` on deduplication (27/40)
+            running `cargo check --no-default-features --features c,d,easytime/std` on deduplication (28/40)
+            running `cargo check --no-default-features --features e,easytime/std` on deduplication (29/40)
+            running `cargo check --no-default-features --features c,e,easytime/std` on deduplication (30/40)
+            running `cargo check --no-default-features --features easytime/default,easytime/std` on deduplication (31/40)
+            running `cargo check --no-default-features --features a,easytime/default,easytime/std` on deduplication (32/40)
+            running `cargo check --no-default-features --features b,easytime/default,easytime/std` on deduplication (33/40)
+            running `cargo check --no-default-features --features c,easytime/default,easytime/std` on deduplication (34/40)
+            running `cargo check --no-default-features --features d,easytime/default,easytime/std` on deduplication (35/40)
+            running `cargo check --no-default-features --features a,d,easytime/default,easytime/std` on deduplication (36/40)
+            running `cargo check --no-default-features --features b,d,easytime/default,easytime/std` on deduplication (37/40)
+            running `cargo check --no-default-features --features c,d,easytime/default,easytime/std` on deduplication (38/40)
+            running `cargo check --no-default-features --features e,easytime/default,easytime/std` on deduplication (39/40)
+            running `cargo check --no-default-features --features c,e,easytime/default,easytime/std` on deduplication (40/40)
             ",
         );
 }
@@ -675,15 +669,14 @@ fn powerset_group_features() {
         .assert_success("real")
         .stderr_contains(
             "
-            running `cargo check --no-default-features` on real (1/9)
-            running `cargo check --no-default-features --features c` on real (2/9)
-            running `cargo check --no-default-features --features default` on real (3/9)
-            running `cargo check --no-default-features --features c,default` on real (4/9)
-            running `cargo check --no-default-features --features a,b` on real (5/9)
-            running `cargo check --no-default-features --features c,a,b` on real (6/9)
-            running `cargo check --no-default-features --features default,a,b` on real (7/9)
-            running `cargo check --no-default-features --features c,default,a,b` on real (8/9)
-            running `cargo check --no-default-features --all-features` on real (9/9)
+            running `cargo check --no-default-features` on real (1/8)
+            running `cargo check --no-default-features --features c` on real (2/8)
+            running `cargo check --no-default-features --features default` on real (3/8)
+            running `cargo check --no-default-features --features c,default` on real (4/8)
+            running `cargo check --no-default-features --features a,b` on real (5/8)
+            running `cargo check --no-default-features --features c,a,b` on real (6/8)
+            running `cargo check --no-default-features --features default,a,b` on real (7/8)
+            running `cargo check --no-default-features --features c,default,a,b` on real (8/8)
             ",
         )
         .stderr_not_contains(
@@ -697,11 +690,10 @@ fn powerset_group_features() {
         .assert_success("real")
         .stderr_contains(
             "
-            running `cargo check --no-default-features` on real (1/5)
-            running `cargo check --no-default-features --features default` on real (2/5)
-            running `cargo check --no-default-features --features a,b,c` on real (3/5)
-            running `cargo check --no-default-features --features default,a,b,c` on real (4/5)
-            running `cargo check --no-default-features --all-features` on real (5/5)
+            running `cargo check --no-default-features` on real (1/4)
+            running `cargo check --no-default-features --features default` on real (2/4)
+            running `cargo check --no-default-features --features a,b,c` on real (3/4)
+            running `cargo check --no-default-features --features default,a,b,c` on real (4/4)
             ",
         )
         .stderr_not_contains(
@@ -725,15 +717,14 @@ fn powerset_group_features() {
     .assert_success("real")
     .stderr_contains(
         "
-        running `cargo check --no-default-features` on real (1/9)
-        running `cargo check --no-default-features --features default` on real (2/9)
-        running `cargo check --no-default-features --features a,b` on real (3/9)
-        running `cargo check --no-default-features --features default,a,b` on real (4/9)
-        running `cargo check --no-default-features --features a,c` on real (5/9)
-        running `cargo check --no-default-features --features default,a,c` on real (6/9)
-        running `cargo check --no-default-features --features a,b,a,c` on real (7/9)
-        running `cargo check --no-default-features --features default,a,b,a,c` on real (8/9)
-        running `cargo check --no-default-features --all-features` on real (9/9)
+        running `cargo check --no-default-features` on real (1/8)
+        running `cargo check --no-default-features --features default` on real (2/8)
+        running `cargo check --no-default-features --features a,b` on real (3/8)
+        running `cargo check --no-default-features --features default,a,b` on real (4/8)
+        running `cargo check --no-default-features --features a,c` on real (5/8)
+        running `cargo check --no-default-features --features default,a,c` on real (6/8)
+        running `cargo check --no-default-features --features a,b,a,c` on real (7/8)
+        running `cargo check --no-default-features --features default,a,b,a,c` on real (8/8)
         ",
     )
     .stderr_not_contains(
@@ -986,7 +977,7 @@ fn each_feature_all() {
 #[test]
 fn include_deps_features() {
     cargo_hack(["check", "--each-feature", "--include-deps-features"])
-        .assert_success2("powerset_deduplication", Some(41))
+        .assert_success2("powerset_deduplication",  Some(if has_stable_toolchain() { 34 } else { 41 }))
         .stderr_contains(
             "
             running `cargo check --no-default-features` on deduplication (1/9)
@@ -1198,11 +1189,12 @@ fn list_separator() {
 fn verbose() {
     cargo_hack(["check", "--verbose"]).assert_success("virtual").stderr_contains(format!(
         "
-        running `cargo check --manifest-path member1{0}Cargo.toml`
-        running `cargo check --manifest-path member2{0}Cargo.toml`
-        running `cargo check --manifest-path dir{0}not_find_manifest{0}Cargo.toml`
+        cargo{1} check --manifest-path member1{0}Cargo.toml`
+        cargo{1} check --manifest-path member2{0}Cargo.toml`
+        cargo{1} check --manifest-path dir{0}not_find_manifest{0}Cargo.toml`
         ",
-        SEPARATOR
+        SEPARATOR,
+        env::consts::EXE_SUFFIX
     ));
 }
 
@@ -1342,3 +1334,23 @@ fn clean_per_version_failure() {
         .assert_failure("real")
         .stderr_contains("--clean-per-version can only be used together with --version-range");
 }
+
+#[test]
+fn keep_going() {
+    cargo_hack(["check", "--each-feature", "--keep-going"])
+        .assert_failure("keep_going")
+        .stderr_contains(format!(
+            "
+            running `cargo check --no-default-features` on keep_going (1/2)
+            `a` feature not specified
+            running `cargo check --no-default-features --features a` on keep_going (2/2)
+            `a` feature specified
+            failed to run 2 commands
+            failed commands:
+            keep_going:
+            cargo{0} check --manifest-path Cargo.toml --no-default-features`
+            cargo{0} check --manifest-path Cargo.toml --no-default-features --features a`
+            ",
+            env::consts::EXE_SUFFIX
+        ));
+}
